// Minify with https://jscompress.com/

var Finder = (function createFinder() {

    function isVisible(element) {
        var isVisible = true;
        var parent = element;
        while (parent && parent.tagName.toLowerCase() != "body" && isVisible) {
            var style = window.getComputedStyle(parent);
            isVisible = style.display != "none" && style.visibility != "hidden" &&
                style.visibility != "collapsed";
            parent = parent.parentNode;
        }
        return isVisible;
    }

    function isOperable(element) {
        return !element.hasAttribute("disabled") && isVisible(element);
    }

    function isInput(element) {
        return element && element.tagName.toLowerCase() == "input";
    }

    function isLoginInput(input) {
        if (input.hasAttribute("type")) {
            var inputType = input.getAttribute("type").toLowerCase();
            return inputType == "text" || inputType == "email";
        }
        return true;
    }

    function isPasswordInput(input) {
        if (input.hasAttribute("type") && input.getAttribute("type").toLowerCase() == "password") {
            return true;
        }
        return false;
    }

    function getAllInputs(element, operableOnly) {
        var allInputs = [];
        var inputs = element.getElementsByTagName("input");
        for (var i = 0; i < inputs.length; i++) {
            var input = inputs.item(i);
            if (!operableOnly || isOperable(input)) {
                allInputs.push(input);
            }
        }
        return allInputs;
    }

    function getPasswordInputs(element, operableOnly) {
        var passwordInputs = [];
        var inputs = element.getElementsByTagName("input");
        for (var i = 0; i < inputs.length; i++) {
            var input = inputs.item(i);
            if (isPasswordInput(input)) {
                if (!operableOnly || isOperable(input)) {
                    passwordInputs.push(input);
                }
            }
        }
        return passwordInputs;
    }

    function getPrevLoginInput(passwordInput, operableOnly) {
        var inputs = getAllInputs(passwordInput.form || document, operableOnly);
        var prevInput = null;
        for (var i = 0; i < inputs.length; i++) {
            if (passwordInput === inputs[i]) {
                return prevInput;
            }
            if (isLoginInput(inputs[i])) {
                prevInput = inputs[i];
            }
        }
        return null;
    }

    function getNextPasswordInput(loginInput, operableOnly) {
        var inputs = getAllInputs(loginInput.form || document, operableOnly);
        var loginPassed = false;
        for (var i = 0; i < inputs.length; i++) {
            if (loginInput === inputs[i]) {
                loginPassed = true;
                continue;
            }
            if (loginPassed && isPasswordInput(inputs[i])) {
                return inputs[i];
            }
        }
        return null;
    }

    function findInputs(operableOnly, form) {
        var loginInput = null, passwordInput = null;
        // has focused input ?
        if (!form && isInput(document.activeElement)) {
            // password ?
            if (isPasswordInput(document.activeElement)) {
                passwordInput = document.activeElement;
                loginInput = getPrevLoginInput(passwordInput, operableOnly);
            } else {
                loginInput = document.activeElement;
                passwordInput = getNextPasswordInput(loginInput, operableOnly);
            }
        }
        // has password input ?
        if (!passwordInput) {
            // try first password
            var passwordInputs = getPasswordInputs(form || document, operableOnly);
            if (passwordInputs.length != 0) {
                passwordInput = passwordInputs[0];
                loginInput = getPrevLoginInput(passwordInput, operableOnly);
            }
        }
        // has any input ?
        if (!passwordInput && !loginInput) {
            var allInputs = getAllInputs(form || document, operableOnly);
            if (allInputs.length != 0) {
                loginInput = allInputs[0];
            }
        }
        return {
            loginInput : loginInput,
            passwordInput : passwordInput,

            getLoginValue : function () {
                return loginInput ? loginInput.value : null;
            },

            getPasswordValue : function () {
                return passwordInput ? passwordInput.value : null;
            }
        };
    }

    return {
        findInputs : function (operableOnly, form) {
            return findInputs(operableOnly, form);
        }
    };

})();

var Filler = (function createFiller() {

    function createKeyEvent(target, type) {
        var event = document.createEvent('KeyboardEvent');
        var initMethod = typeof event.initKeyboardEvent !== 'undefined' ? "initKeyboardEvent" : "initKeyEvent";
        event[initMethod](
                    type, // event type : keydown, keyup, keypress
                    true, // bubbles
                    false, // cancelable
                    null, // viewArg: should be window
                    false, // ctrlKeyArg
                    false, // altKeyArg
                    false, // shiftKeyArg
                    false, // metaKeyArg
                    0, // keyCodeArg : unsigned long the virtual key code, else 0
                    0 // charCodeArgs : unsigned long the Unicode character associated with the depressed key, else 0
        );
        return event;
    }

    function createEvent(type) {
        var event = document.createEvent("HTMLEvents");
        event.initEvent(type, true, true);
        return event;
    }

    function fakeKeyEvent(input) {
        input.dispatchEvent(createKeyEvent(input, "keydown"));
        input.dispatchEvent(createKeyEvent(input, "keypress"));
        input.dispatchEvent(createKeyEvent(input, "keyup"));
    }

    function fillInput(input, value) {
        if (input) {
            // prepare
            input.click();
            input.focus();
            fakeKeyEvent(input);
            // set
            input.value = value;
            // flush
            fakeKeyEvent(input);
            input.dispatchEvent(createEvent("input"));
            input.dispatchEvent(createEvent("change"));
            // just in case
            if (input.value != value) {
                input.value = value;
            }
        }
    }

    return {
        fillInput: function(input, value) {
            return fillInput(input, value);
        }
    };
})();

function isInput(element) {
    return element && element.tagName.toLowerCase() == "input";
}

var res = "false";
var inputs = Finder.findInputs(true);

if (aCommand == "paste_both") {
    // fill login/password
    if (inputs.loginInput || inputs.passwordInput) {
        Filler.fillInput(inputs.loginInput, aLogin);
        Filler.fillInput(inputs.passwordInput, aPassword);
        res = "true";
    }
} else if (aCommand == "silent_paste") {
    // only if password found
    if (inputs.passwordInput) {
        Filler.fillInput(inputs.loginInput, aLogin);
        Filler.fillInput(inputs.passwordInput, aPassword);
        res = "true";
    }
} else if (aCommand == "paste_login") {
    if (isInput(document.activeElement)) {
        Filler.fillInput(document.activeElement, aLogin);
        res = "true";
    }
} else if (aCommand == "paste_password") {
    if (isInput(document.activeElement)) {
        Filler.fillInput(document.activeElement, aPassword);
        res = "true";
    }
} else if (aCommand == "paste_one_time_password") {
    if (isInput(document.activeElement)) {
        Filler.fillInput(document.activeElement, aPasscode);
        res = "true";
    }
}
